class Moon < ActiveRecord::Base
  has_one :fleet
  belongs_to :player
  validates_numericality_of :amount

  # Is this a home moon?
  def is_home_moon?
    home_moon == 'true' ? true : false
  end
  
  # How much money do you get from this moon per round
  def taxes
    size.to_i * 5
  end
  
  # Retruns the strength of the fleet + strength of all satellites on the moon
  def total_strength
    self.fleet.total_strength + self.satellite_count*2
  end
  
  # Resets the production to money
  def reset_production
    self.product, self.amount, self.rounds_left = 'money', 1, 1
    self.save
  end
  
  # Cancels the current production and resets it back to money
  def cancel_production
    case self.product
    when 'money':
      strength = 0
    when 'stalitte'
      strength = 1
    else
      strength = self.product
    end
    
    unless self.player.nil?
      self.player.money += ( 0.75 * ( self.amount.to_i * Spaceship.get_price(strength) ) ).to_i
      self.player.save
    end
    self.reset_production
  end
  
  # How many rounds passed since the last update of the player
  def rounds_passed
    self.last_update_on = self.player.last_update_on if self.last_update_on.nil? # Set last_update_on of the moon to the players value if the moon has never been updated
    ((Time.now-self.last_update_on)/Game.round_length).to_i
    # <-- Fix RadRails syntax highlighting bug
  end
  
  # Calculates the distance to another moon
  def distance_to(moon)
  	(Math::sqrt((x_pos-moon.x_pos)*(x_pos-moon.x_pos)+(y_pos-moon.y_pos)*(y_pos-moon.y_pos))).round
  end
  
  # How long will it take to reach another moon
  def time_to_reach_from(moon)
    (1.5*distance_to(moon)+0.5).round
  end

  # How many ships can be produced for the amount of rounds passed
  def capacity_for_passed_rounds
    (((self.rounds_needed_to_produce_ship - self.rounds_left) + self.rounds_passed) / self.rounds_needed_to_produce_ship ).to_i
    # <-- A comment to fix RadRails syntax highlighting error ^^
  end
  
  # How long does the moon need to produce a certain or the current ship in production
  def rounds_needed_to_produce_ship(strength=nil)
    if strength.nil?
      case self.product.to_s
      when 'satellite'
        return 6-self.size.to_i
      else
        return (self.product.to_i+1)+(4-self.size.to_i)
      end
    else
      return (strength.to_i+1)+(4-self.size.to_i)
    end
  end

  # Creates a new fleet if the moon has none yet
  def create_fleet
    if self.fleet.nil?
      self.fleet = Fleet.new
      
      case self.size.to_i
      when 1:
        self.fleet.weak_ships = 3
        self.fleet.medium_ships = 2
        self.fleet.strong_ships = 1
      when 2:
        self.fleet.weak_ships = 3
        self.fleet.medium_ships = 3
        self.fleet.strong_ships = 2
      when 3:
        self.fleet.weak_ships = 3
        self.fleet.medium_ships = 3
        self.fleet.strong_ships = 3
      end
      
      self.fleet.save
    end
  end

  # Produces a certain amount of ships
  def produce_ships(amount_to_produce)
    create_fleet
    
    case self.product.to_s
    when '1'
      self.fleet.weak_ships += amount_to_produce
    when '2'
      self.fleet.medium_ships += amount_to_produce
    when '3'
      self.fleet.strong_ships += amount_to_produce
    when 'satellite'
      self.satellite_count += amount_to_produce
    end
    
    if amount_to_produce.zero?
      self.rounds_left -= self.rounds_passed
    else
      self.rounds_left = rounds_needed_to_produce_ship-(self.rounds_passed-self.rounds_left)%rounds_needed_to_produce_ship
    end
        
    self.fleet.save
    self.amount -= amount_to_produce
    if (self.amount.zero?)
      self.player.money += self.taxes * (capacity_for_passed_rounds - amount_to_produce) * rounds_needed_to_produce_ship
      self.player.save
      self.reset_production
    end
    self.last_update_on += Game.round_length*self.rounds_passed
  end
  
  # Creates a new moon :) In the MoWaon galaxy, this is possible
  def self.make
    m = Moon.new
    srand
    begin
      m.x_pos,m.y_pos = rand(Galaxy.boundry)+1,rand(Galaxy.boundry)+1
    end until Moon.find(:all,:conditions => "x_pos >= #{m.x_pos-1} AND x_pos <= #{m.x_pos+3} AND y_pos >= #{m.y_pos-1} AND y_pos <= #{m.y_pos+1}").empty?
    m.size = ((rand*150).to_i%3+1).to_s
    m.name = self.new_name
    m.reset_production
    m.create_fleet
    m.save
    m
  end
  
  # Creates a random name for the moon
  def self.new_name(fix_count=3)
    begin
      fixes = %w( ba be bi bo bu ta te ti to tu na ne ni no nu ka ke ki ko ku fa fe fi fo fu bla ug ju ja ji jo je on da po pe pal ox ux puz zu za ze )
      name = ""
      fix_count.times { name += fixes[rand(fixes.size)] }
      name = name.capitalize
    end until Moon.find(:all,:conditions => "name = '#{name}'").empty?
    name
  end

  protected
  
  def validate
    errors.add(:amount, "should be an integer") unless amount.is_a?Fixnum
    errors.add(:amount, "should be positive") unless amount.nil? || amount > 0
  end
  
end
